@@ -17,11 +17,11 @@ class RequestHeaders(object):
 
 
 class HttpRequest():
-    def __init__(self, scope, body, receive):
+    def __init__(self, scope, body, receive, metadata=None):
         self._scope = scope
         self._body = body
         self._receive = receive
-        self._headers = None
+        self._subdomain, self._headers = metadata
         self._params = None
 
     def _parse_qs(self):
@@ -68,6 +68,10 @@ class HttpRequest():
     def querystring(self):
         return self._scope.get('query_string', {})
     
+    @property
+    def subdomain(self):
+        return self._subdomain
+
     @property
     def url(self):
         return self._scope.get('path')
