@@ -1,10 +1,10 @@
 from unittest import TestCase
 
-from routerling import Router
-from routerling import Context
-from routerling import HttpRequest, ResponseWriter
+from . import Router
+from . import Context
+from . import HttpRequest, ResponseWriter
 
-from routerling.router import Routes
+from .router import Routes
 
 
 # to be reused in router testcase instance
@@ -21,10 +21,10 @@ class TestRouter(TestCase):
         ))
     
     def test_route_instance_found(self):
-        self.assertIsInstance(self.router.rtx, Routes)
+        self.assertIsInstance(self.router.subdomains.get('www'), Routes)
 
     def test_index_get_node(self):
-        methods = self.router.rtx.routes
+        methods = self.router.subdomains.get('www').routes
         self.assertEqual("nothing at all", "nothing at all")
 
 
